// functions
// --------------------------
@use "sass:string";

// fa-content: convenience function used to set content property
@function fa-content($var) {
  @return string.unquote("\"#{ $var }\"");
}
